// |$Id: setstream.c,v 1.1 2024/03/11 22:48:06 pm Exp $     |
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              |
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS    http://173.255.241.228              | 
   | FUNCTION NAME: setstream.c                                 |
   | DIRECTORY: bsu-3.0.3/src/C/bsegy                           |
   |------------------------------------------------------------|
   | purpose: Set land streamer geometry                        |
   |          Assumes basic headers on BSEGY data, but fixes    |
   |          source and receiver locations                     |
   |------------------------------------------------------------|
   |  NOTICE:                                                   |
   |  Copyright (C) 2021 Paul Michaels                          |
   |  <paulmichaels@boisestate.edu>                             |
   |  This program is free software; you can                    |
   |  redistribute it and/or modify it under the terms          |
   |  of the GNU General Public License as published            |
   |  by the Free Software Foundation; either version           |
   |  3 of the License, or (at your option) any later           |
   |  version.  This program is distributed in the              |
   |  hope that it will be useful, but                          |
   |  WITHOUT ANY WARRANTY; without even the implied            |
   |  warranty of MERCHANTABILITY or FITNESS FOR A              |
   |  PARTICULAR PURPOSE.  See the GNU General Public           |
   |  License for more details.                                 |
   |  You should have received a copy of the GNU                |
   |  General Public License along with this program;           |
   |  if not, write to the Free Software Foundation,            |
   |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
   |------------------------------------------------------------|
   |INLINE FUNCTIONS:                                           |
   |  getparm    get parameters from command line or interactive|
   |  outlst     echo check of input parameters to listing file |
   |                                                            |
   |------------------------------------------------------------|
   |sublibC4 functions:                                         |
   |  c_bsegin   read seismic trace                             |
   |  c_bsegout  write a seismic trace                          |
   |  in_chk     scan input file for samples, number traces etc.|
   |  bargrid    write progress bar grid to screen              |
   |  exbar      write progress bar to screen                   |
   |  cr_labl    write copyright label to screen                |
   |  fcr_labl   write copyright label to file                  |
   |  names      create an output file name from input file name|
   |                                                            |
   |------------------------------------------------------------|
*/
      #include <stdio.h>
      #include <stdlib.h>
      #include <float.h>
      #include <math.h>
      #include <string.h>
      #include <time.h>
      #include "../include/c_bsegy.h"

//...parameter definitions

//...function prototypes
      #include "../include/sub4.h"

      int  getparm(int argc,char *argv[],
         float *Sx, float *Sy, float *Sz, int *Nch, float *Ggap, float *Gint,
         float *Az, char infile[], char ofile[], char lstfile[],char pid[]);

      void outlst(float Sx, float Sy, float Sz, int Nch, float Ggap, float Gint, float Az,
	char infile[], char pid[], char ofile[], char lstfile[], FILE *h3);


      int  main(int argc, char *argv[])
     {

//...declare variables
      FILE              *h1, *h2, *h3;
      int               jt,
                        jrec,
                        npts,
                        ntraces,
                        ibar,
                        invbar;
      float             fsamin,
                        bar,
                        *s1,
                        *s2;
      char              pid[5]="sets",   //process id name (4char)
                        ofile[13],
                        lstfile[13],
                        infile[14];
      struct header     hd;

//...specific to this application
      int               Nch;   // number of channels
      float             Sx;    // source X-coord
      float             Sy;    // source Y-coord
      float             Sz;    // source Elevation
      float             Ggap;  // gap to near phone
      float             Gint;  // interval between phones
      float             Az;    // azimuth of cable from source
	int		filenumber; //header ofrn = file record name
	int		rstation;   // receiver station name
	char	buff[10];
	float		frange;
	float		PI=3.141592653589793;
	float		angle,rx,ry;



//...get parameters
      if(getparm(argc,argv,&Sx,&Sy,&Sz,&Nch,&Ggap,&Gint,&Az,
         infile,ofile,lstfile,pid)!=0)
      {
       goto quick_exit;
      }

//...open output listing file
        if( (h3=fopen(lstfile,"w"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s",lstfile);
          goto quick_exit;
        }

//...echo check to listing file
      outlst(Sx,Sy,Sz,Nch,Ggap,Gint,Az,infile,pid,ofile,lstfile,h3);

//...open input file
        if( (h1=fopen(infile,"rb"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s",infile);
          goto exit_3;
        }

//...open output file
        if( (h2=fopen(ofile,"wb"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s",ofile);
          goto exit_1;
        }


//...check input file size
          if(in_chk(&ntraces,&npts,&fsamin,hd,h1)!=0)
         {
          goto exit_2;
         }

//...print input file data to lstfile
        fprintf(h3,"\n     Number of samples= %d",npts);
        fprintf(h3,"\n     Sample Interval = %5.4f sec.",fsamin);
        fprintf(h3,"\n     Number of traces = %d \n",ntraces);
fflush(h3);

//...parameter check
	if (Nch != ntraces) {
	fprintf(stderr,"ABORT: number of channels error \n");
	goto exit_3;}

	if (Az > 359.0 ) {
	fprintf(stderr,"ABORT: Azimuth error \n");
	goto exit_3;}

	if (Ggap <0. || Gint <0. ) {
	fprintf(stderr,"ABORT: Gap or interval error \n");
	goto exit_3;}

	if (fabs(Sx)>32768 ) {
	fprintf(stderr,"ABORT: Sx size too large \n");
	goto exit_3;}

	if (fabs(Sy)>32768 ) {
	fprintf(stderr,"ABORT: Sy size too large \n");
	goto exit_3;}



//...dynamic allocation of seimic trace memory
        s1=calloc(npts,sizeof(float));
          if(s1==NULL)
         {
          fprintf(stderr,"ABORT:s1 can't allocate enough trace memory");
          fprintf(stderr," npts = %d",npts);
          goto exit_2;
         }
        s2=calloc(npts,sizeof(float));
          if(s2==NULL)
         {
          fprintf(stderr,"ABORT:s2 can't allocate enough trace memory");
          fprintf(stderr," npts = %d",npts);
          goto exit_s1;
         }
//-----------------main loop---------------------------
//...display progress bar grid

      bargrid(ntraces,&bar,&ibar,&invbar);

//...trace loop

      for (jrec=0;jrec<ntraces;jrec++)
     {

//...read trace
        if(c_bsegin(jrec,s1,npts,&hd,h1)!=0)
       {
        fprintf(stderr,"ABORT--c_bsegin");
        goto quit_it;
       }

//...Update Header info
	(hd).smul1 = (short int) -1000;
	(hd).smul2 = (short int) -1000;
	(hd).ishlo = (int ) (Sx*1000.);
	(hd).ishla = (int ) (Sy*1000.);
	(hd).ishe  = (int ) (Sz*1000.);
	(hd).irel  = (int ) (Sz*1000.); //set geoph elevations at source elevation
	filenumber=(hd).ofrn;
	sprintf(buff,"%4.4d",filenumber);
	memcpy(&(hd).spname[0],&buff[0],4);	//shot point name
	rstation =filenumber - ((int) Ggap/Gint + jrec);
	sprintf(buff,"%4.4d",rstation);
	memcpy(&(hd).rstnam[0],&buff[0],4);	//receiver site name
	frange = Ggap + ((float) jrec) * Gint;
	(hd).idist = (int) frange;
	angle = (90.0-Az)*(PI/180.);
	rx = frange*cos(angle) + Sx;
	ry = frange*sin(angle) + Sy;
	(hd).irlo = (int) (rx*1000.);
	(hd).irla = (int) (ry*1000.);



//...output trace
        if(c_bsegout(jrec,s1,npts,&hd,h2)!=0)
       {
        fprintf(stderr,"ABORT--c_bsegout");
        goto quit_it;
       }

//...display progress bar
      exbar(ntraces,bar,ibar,invbar,jrec+1);

     } //  end trace loop-----------------------------

      printf("\n   Listing ====>%s",lstfile);
      printf("\n   Output  ====>%s",ofile);
      printf("\n   Processed %d traces \n",ntraces);
	free(s1);
	free(s2);
        return(0);

quit_it:
        puts(" ");
        free(s2);
exit_s1:
        free(s1);
exit_2:
        fclose(h2);
exit_1:
        fclose(h1);
exit_3:
        fclose(h3);
quick_exit:
        return(1);
     } //  end of main
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <paulmichaels@boisestate.edu>          |
   | FUNCTION NAME: getparm.c                                   |
   | Copyright (C) 2021 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: gets input parameters from command                |
   |          line or prompts user from consol.                 |
   |                                                            |
   |                                                            |
   |------------------------------------------------------------|
*/
      int  getparm(int argc,char *argv[],
         float *Sx, float *Sy, float *Sz, int *Nch, float *Ggap, float *Gint,
         float *Az, char infile[], char ofile[], char lstfile[],char pid[])
     {
      char              sfx1[4]="seg",
                        sfx2[4]="lst";
      char		help[3]="-h";
      char              msg1[46],msg2[46];

//...test for command line help ?
      if(argc==2 && !strcmp(argv[1],help))
     {
//...screen header and copyright notice
      cr_labl(pid);

//...help pannel
      sprintf(msg1,"Set Land Streamer Geometry                   ");
      sprintf(msg2,"Takes existing BSEGY headers and fixes       ");
      hlp_labl(msg1,msg2);
      puts(" ");

//...help display
      puts(" ");
      printf("  %stream  infile  Sx Sy Sz Nch Ggap Gint Az \n",pid);
      puts(" ");
      puts("     infile  = input file name (4char minimum)");
      puts(" ");
      puts("     Sx      = source x-coordinate       ");
      puts("     Sy      = source y-coordinate       ");
      puts("     Sz      = source z-coordinate       ");
      puts("     Nch     = number of channels        ");
      puts("     Ggap    = gap to first geophone     ");
      puts("     Gint    = interval between phones   ");
      puts("     Az      = azimuth of cable from source (N=0 deg)");
      puts("               West=270 deg              ");
      puts(" ");
      return(1);
     }
      else
     {
//...get parameters------------------------------
//...pars off character parameters
      if(argc>1)
       strncpy(infile,argv[1],13);
       else
      {
       puts(" Enter INPUT file name ");
       scanf(" %13s",infile);
      }
	if (namecheck(12,infile) != 0 ) return (1);

//...convert string to float  
      if(argc>2)
       *Sx=atof(argv[2]);
       else
      {
       puts(" Enter Sx (float)");
       scanf(" %f",Sx);
      }

//...convert string to float
      if(argc>3)
       *Sy=atof(argv[3]);
       else
      {
       puts(" Enter Sy (float)");
       scanf(" %f",Sy);
      }

//...convert string to float
      if(argc>4)
       *Sz=atof(argv[4]);
       else
      {
       puts(" Enter Sz (float)");
       scanf(" %f",Sz);
      }


//...convert string to integer
      if(argc>5)
       *Nch=atoi(argv[5]);
       else
      {
       puts(" Enter Nch = number of channels (int)");
       scanf(" %d",Nch);
      }

//...convert string to float
      if(argc>6)
       *Ggap=atof(argv[6]);
       else
      {
       puts(" Enter Ggap = source to geophone gap (float)");
       scanf(" %f",Ggap);
      }

//...convert string to float
      if(argc>7)
       *Gint=atof(argv[7]);
       else
      {
       puts(" Enter Gint = interval between phones (float)");
       scanf(" %f",Gint);
      }

//...convert string to float
      if(argc>8)
       *Az=atof(argv[8]);
       else
      {
       puts(" Enter Az = Azimuth of cable from source (float)");
       scanf(" %f",Az);
      }

//...name output files
      names(infile,pid,ofile,sfx1);
      names(infile,pid,lstfile,sfx2);
       return(0);
     }  //  endif bhelp test
      } // end getparm
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <paulmichaels@boisestate.edu>          |
   | FUNCTION NAME: outlst.c                                    |
   | Copyright (C) 2021 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: prints copyright header, file names, and          |
   |          echo check of input parameters in newly           |
   |          opened output listing file.                       |
   |                                                            |
   |------------------------------------------------------------|
*/
      void outlst(float Sx, float Sy, float Sz, int Nch, float Ggap, float Gint, float Az,
	char infile[], char pid[], char ofile[], char lstfile[], FILE *h3)
     {

//...print copyright label to outlst and screen

      fcr_labl(pid,h3);

//...echo check
      fputs("\n",h3);                      //blank line
      fprintf(h3,"\n     PROGRAM: %stream.c ",pid);
      fputs("\n",h3);                      //blank line
      fputs("\n     FILES:       ",h3);
      fprintf(h3,"\n     Input   File= %s",infile);
      fprintf(h3,"\n     Output  File= %s",ofile);
      fprintf(h3,"\n     Listing File= %s",lstfile);
      fputs("\n",h3);                      //blank line
      fputs("\n     INPUT PARAMETERS: ",h3);
      fprintf(h3,"\n     Sx = %.2f       ",Sx);
      fprintf(h3,"\n     Sy = %.2f       ",Sy);
      fprintf(h3,"\n     Sz = %.2f       ",Sz);
      fprintf(h3,"\n     Nch= %d         ",Nch);
      fprintf(h3,"\n     Ggap = %.2f       ",Ggap);
      fprintf(h3,"\n     Gint = %.2f       ",Gint);
      fprintf(h3,"\n     Az = %.2f       ",Az);
      fputs("\n",h3);                      //blank line
     }  //  end outlst
