// |$Id: bwfi.c,v 1.43 2024/06/10 23:10:39 pm Exp $          |
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              |
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS    http://173.255.241.228              | 
   | FUNCTION NAME: bwfi.c                                      |
   | DIRECTORY: bsu-3.0.3/src/C/bsegy                           |
   |------------------------------------------------------------|
   | purpose: Wave Field inversion of Rayleigh wave data        |
   |------------------------------------------------------------|
   |  NOTICE:                                                   |
   |  Copyright (C) 2021 Paul Michaels                          |
   |  <paulmichaels@boisestate.edu>                             |
   |  This program is free software; you can                    |
   |  redistribute it and/or modify it under the terms          |
   |  of the GNU General Public License as published            |
   |  by the Free Software Foundation; either version           |
   |  3 of the License, or (at your option) any later           |
   |  version.  This program is distributed in the              |
   |  hope that it will be useful, but                          |
   |  WITHOUT ANY WARRANTY; without even the implied            |
   |  warranty of MERCHANTABILITY or FITNESS FOR A              |
   |  PARTICULAR PURPOSE.  See the GNU General Public           |
   |  License for more details.                                 |
   |  You should have received a copy of the GNU                |
   |  General Public License along with this program;           |
   |  if not, write to the Free Software Foundation,            |
   |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
   |------------------------------------------------------------|
   |INLINE FUNCTIONS:                                           |
   |  getparm    get parameters from command line or interactive|
   |  outlst     echo check of input parameters to listing file |
   |                                                            |
   |------------------------------------------------------------|
   |sublibC4 functions:                                         |
   |  c_bsegin   read seismic trace                             |
   |  c_bsegout  write a seismic trace                          |
   |  in_chk     scan input file for samples, number traces etc.|
   |  bargrid    write progress bar grid to screen              |
   |  exbar      write progress bar to screen                   |
   |  cr_labl    write copyright label to screen                |
   |  fcr_labl   write copyright label to file                  |
   |  names      create an output file name from input file name|
   |                                                            |
   |------------------------------------------------------------|
*/
      #include <stdio.h>
      #include <stdlib.h>
      #include <float.h>
      #include <math.h>
      #include <string.h>
      #include <time.h>
      #include "../include/c_bsegy.h"

//...parameter definitions

//...function prototypes
      #include "../include/sub4.h"

	int CalcAlphaMu(double Vs,double Den, double VpVs, 
	 double *Vp, double *Mu, double *Lame );

      int  getparm(int argc,char *argv[],
	double *Vo, double *Vmn, double *Vmx, int *NV, double *Zmn,
	double *Zmx, int *NZ, double *Vhsp, double *Zhsp, double *VpVs, double *Rho, double *deltz,
	double *segsw, double *fmin, double *fmax, double *firvsel,
	float *tmin, float *tmax, char infile[], char ofile[],
	char lstfile[],char pid[]);

      void outlst(double Vo, double Vmn, double Vmx, int NV, double Zmn,
	double Zmx, int NZ, double Vhsp, double Zhsp, double VpVs,
	double Rho, double deltz, double segsw, 
	double fmin, double fmax, double firvsel, float tmin, float tmax,
	short obselect, char infile[], char pid[], char ofile[],
	char lstfile[], FILE *h3);

      void rwv_(int *nlay, double rho[], double mu[], double lame[],
	double zi[], double *deltz, double *freq, int *maxmod, double pvf[]);

      void wavsub_(int *nfreq, double Freq[], double pv[], double *fsamin2, double *firvsel,
	double sxyz[],int *nrec, double Rx[], double Ry[], double Rz[], int *nlay,
	double rho[], double mu[], double lame[], double zi[], int *Nxout,
	double xout[], double *segsw, int *IDcase);

      void DisperdOut(double svrho[], double svmu[], double svlame[],
	double svzi[], double deltz, int Nfreq, double fmin, double delf,int mincase);

      void  ModelOut(double svBeta[], double svzi[], int mincase);

	void  WavesdOut(int ntraces, double Rx[], double Ry[], double Rz[],
	double Sx, double Sy, double Sz,
	double fmin, double fmax, float fsamin, double firvsel, 
	int mincase, int stepz);

	void ContourPlot(FILE * h5, int mincase, int maxcase, int Ncases,
	double savmean[], double savstdv[], double savKZ[], double savJV[],
	double Zmn, double Zmx,
	double OBJF[], int NZ, char infile[],int argc,char *argv[]);

	void	GridPoints(FILE * h5, int mincase, int maxcase, int Ncases, 
	double savKZ[], double savJV[], double Vo, double Vmn, double Vmx, int NV,
	double Zmn, double Zmx, int NZ, double Vhsp, double Zhsp, double DV, double DZ, 
	char infile[],int argc,char *argv[]);


      int  main(int argc, char *argv[])
     {

//...declare variables
      FILE              *h1, *h2, *h3, *h5;
      int		stepz;
      int               jt,
                        jrec,
                        npts,
                        ntraces,
                        ibar,
                        invbar;
      float             fsamin,
                        bar,
                        *s1,
                        *s2,
			*tpoint;
      char              pid[5]="bwfi",   //process id name (4char)
                        ofile[13],
                        lstfile[13],
                        infile[14];
      struct header     hd;

//...specific to this application

	int		nlay,
			maxmod;
	double		rho[3],
			mu[3],
			lame[3],
			zi[3],
			Beta[3],
			Alpha[3],
			svrho[3],  //sv instance for minimum obj. func.
			svmu[3],
			svzi[3],
			svlame[3],
			svBeta[3],
			pvf[9],
			pv[5000];
	double		deltz,
			fsamin2,
			segsw,
			firvsel,
			freq;
	float		sx,sy,sz,sd,rx,ry,rz,range;
	double		Rx[200], Ry[200], Rz[200];
	double		sxyz[3];
	int		Nxout;
	double		*xout;  // pointer to synthetic traces
	int		DEBUG0=0;  // set to 1 for dispersion listing
	int		DEBUG1=0;  // set to 1 for source and receiver listing
	int		IDcase=-1,
			jcase;
	int		nrec;
	int		Nfreq;
	double 		fmin; //set spectrum
	double 		fmax;
	double 		*Freq;
	double 		delf;
	int		indx;
	int 		mode;
	int 		npr2;
	double		DV, DZ; // velocity grid
	int		j,k,m;
	int		Ncases;
	double		iprodsum,
			s1norm,
			xoutnorm,
			cosangle,
			angle;
	int		indexj,
			index2;
	double		PI=3.14159265358979311599796346854;
	double		*savangle,
			*savmean,
			*savstdv;
	double		*savKZ,
			*savJV;
	float 		*savKMXdif;
	double		meanAngle,
			DELT,
			stdevAngle;
	double		minobjf=1.E+10;
	double		maxobjf=-1.E+10;
	int		mincase,
			maxcase;
	int		jmin,  //limits on window for vector space
			jmax;  // sets dimension and what to innerproduct
	float 		tmin,
			tmax;
	float		tmax2;
	double		*OBJF; //new definition of objective function to correct for 90 degrees
	short		obselect = 1; // 0=use stdev  1=use 90 deg correction
	float		FMAX, FMIN;
	int		isw1 = 1; //switch to find max and min absolute values fmax_min function
	int		KMX, KMN;
	float		*kmaxs1, *kmaxdif;
	float		meanKmaxdif;
	int		jt2;

//input variables
	double Vo, Vmn, Vmx, Zmn, Zmx, Vhsp, Zhsp, VpVs, Rho;
	int   NV, NZ;

//...get parameters
      if(getparm(argc,argv,
     	&Vo,&Vmn,&Vmx,&NV,&Zmn,&Zmx,&NZ,
	&Vhsp,&Zhsp,&VpVs,&Rho,&deltz,&segsw,&fmin,&fmax,&firvsel,&tmin,&tmax,  
	infile,ofile,lstfile,pid)!=0)
      {
       goto quick_exit;
      }

//...check and make sure Vhsp is fastest shear velocity in soil
	if (Vmn > Vmx ) {
	fprintf(stderr,"ABORT: Vmx should be greater than Vmn \n");
	fprintf(stderr," Vmn=%f  Vmx=%f \n",Vmn,Vmx);
	abort();
	} //endif
	if (Vmx > Vhsp) {
	fprintf(stderr,"ABORT: Vhsp must be greater than Vmx \n");
	fprintf(stderr,"Vmx=%f  Vhsp=%f ",Vmx,Vhsp);
	abort();
	} //endif
//...check and make sure Zhsp is deepest point in model
	if (Zmx > Zhsp) {
	fprintf(stderr,"ABORT: Zhsp must be greater than Zmx \n");
	fprintf(stderr,"Zmx=%f  Zhsp=%f ",Zmx,Zhsp);
	abort();
	} //endif
//...check if Zmx == Zhsp
	if (Zmx == Zhsp) {
	Zhsp = Zhsp +.01;
	} //endif

//...open output listing file
        if( (h3=fopen(lstfile,"w"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s",lstfile);
          goto quick_exit;
        }

//...echo check to listing file
      outlst(Vo,Vmn,Vmx,NV,Zmn,Zmx,NZ,Vhsp,Zhsp,VpVs,Rho,deltz,segsw,fmin,fmax,firvsel,
	tmin,tmax,obselect,infile,pid,ofile,lstfile,h3);

//...build grid
	DV=(Vmx-Vmn)/(double) NV;
	DZ=(Zmx-Zmn)/(double) NZ;
	fprintf(h3,"DV=%lf  DZ=%lf \n \n",DV,DZ);
	nlay = 3;
	maxmod = 9;
	Ncases = (NV+1)*(NZ+1);
	fprintf(h3,"  Ncases = %d \n",Ncases);
//DEBUG
fprintf(stderr,"Ncases=%d \n",Ncases);
//...display progress bar grid
      bargrid(Ncases,&bar,&ibar,&invbar);
fprintf(h3,"bar= %f  ibar= %d  invbar =%d \n",bar,ibar,invbar);

	/*           FIELD DATA      */
//=============================================================================
//...open input file
        if( (h1=fopen(infile,"rb"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s",infile);
          goto exit_3;
        }

//...check input file size
          if(in_chk(&ntraces,&npts,&fsamin,hd,h1)!=0)
         {
          goto exit_2;
         }
	fsamin2=fsamin;

//...check to see if command line tmax parameter is less than or equal to
//   actual data tmax
	float 	tmaxcheck;
	tmaxcheck = npts*fsamin;
	if (tmax > tmaxcheck) {
	tmax = tmaxcheck;
	fprintf(h3,"\n WARNING: command line parameter tmax reset to that of field data");
        fprintf(h3,"\n     tmax =  %f",tmax);
 	} //endif


//...print input file data to lstfile
        fprintf(h3,"\n     Number of samples= %d",npts);
        fprintf(h3,"\n     Sample Interval = %5.4f sec.",fsamin);
        fprintf(h3,"\n     Number of traces = %d \n",ntraces);
fflush(h3);

//...allocate saved angle for objective function
	savangle=calloc(ntraces,sizeof(double));
	if (savangle==NULL)
	{
          fprintf(stderr,"ABORT:savangle can't allocate enough memory");
	  goto exit_2;
	} //endif

//...allocate saved angle for objective function
	savmean=calloc(Ncases,sizeof(double));
	if (savmean==NULL)
	{
          fprintf(stderr,"ABORT:savmean can't allocate enough memory");
	  goto exit_2;
	} //endif

//...allocate saved stdev for objective function
	savstdv=calloc(Ncases,sizeof(double));
	if (savstdv==NULL)
	{
          fprintf(stderr,"ABORT:savstdv can't allocate enough memory");
	  goto exit_2;
	} //endif

//...allocate saved objective function 
	OBJF=calloc(Ncases,sizeof(double));
	if (OBJF==NULL)
	{
          fprintf(stderr,"ABORT:OBJF can't allocate enough memory");
	  goto exit_2;
	} //endif

//...allocate saved function of maximum arrival sample numbers 
	kmaxs1=calloc(npts,sizeof(float));
	if (kmaxs1==NULL)
	{
          fprintf(stderr,"ABORT:kmaxs1 can't allocate enough memory");
	  goto exit_2;
	} //endif

//...allocate saved function of maximum arrival sample number differences 
	kmaxdif=calloc(npts,sizeof(float));
	if (kmaxdif==NULL)
	{
          fprintf(stderr,"ABORT:kmaxdif can't allocate enough memory");
	  goto exit_2;
	} //endif



//...allocate saved depth for objective function
	savKZ=calloc(Ncases,sizeof(double));
	if (savKZ==NULL)
	{
          fprintf(stderr,"ABORT:savKZ   can't allocate enough memory");
	  goto exit_2;
	} //endif

//...allocate saved velocity for objective function
	savJV=calloc(Ncases,sizeof(double));
	if (savJV==NULL)
	{
          fprintf(stderr,"ABORT:savJV   can't allocate enough memory");
	  goto exit_2;
	} //endif

//...allocate saved difference in max sample arrival time for objective function
	savKMXdif=calloc(Ncases,sizeof(float));
	if (savKMXdif==NULL)
	{
          fprintf(stderr,"ABORT:savKMXdif   can't allocate enough memory");
	  goto exit_2;
	} //endif

//...dynamic allocation of seimic trace memory
        s1=calloc(ntraces*npts,sizeof(float));
          if(s1==NULL)
         {
          fprintf(stderr,"ABORT:s1 can't allocate enough trace memory");
          fprintf(stderr," npts = %d  ntraces=%d \n",npts,ntraces);
          goto exit_2;
         }

//...dynamic allocation of seimic trace memory
        s2=calloc(npts,sizeof(float));
          if(s2==NULL)
         {
          fprintf(stderr,"ABORT:s2 can't allocate enough trace memory");
          fprintf(stderr," npts = %d  \n",npts);
          goto exit_2;
         }


//...Radix 2 for synthetic number of samples
	npr2=2;
	for (jt=0; jt<50; jt++){
		if (npr2>=2*npts) {
		break; 
		}
		else {
		npr2=npr2*2;
		}
	} //next jt

//...dynamic allocation of seimic trace memory synthetics
         xout=calloc(ntraces*npr2,sizeof(double));
          if(xout==NULL)
         {
          fprintf(stderr,"ABORT:xout can't allocate enough trace memory");
          fprintf(stderr," npr2 = %d  ntraces=%d \n",npr2,ntraces);
          goto exit_2;
         }

//...trace loop

      for (jrec=0;jrec<ntraces;jrec++)
     {

//...set trace index pointer for field data
	tpoint = s1 + npts*jrec;

//...read trace
        if(c_bsegin(jrec,tpoint,npts,&hd,h1)!=0)
       {
        fprintf(stderr,"ABORT--c_bsegin");
        goto quit_it;
       }

//...select header coordinates
	findxyz(&sx,&sy,&sz,&sd,&rx,&ry,&rz,&range,hd);
	if (DEBUG1==1) {
	fprintf(stdout,"source: (%f, %f, %f)\n Receiver: ( %f, %f, %f) \n",
	  sx,sy,sz,rx,ry,rz);
	} //endif

//...load cooridnates into arrays
	Rx[jrec]=rx;
	Ry[jrec]=ry;
	Rz[jrec]=rz;
	if (jrec == 0 ) {
	sxyz[0]=sx;
	sxyz[1]=sy;
	sxyz[2]=sz;
	} //endif

//...check that only one source location
	if (jrec > 0) {
		if (sxyz[0]+sxyz[1]+sxyz[2] != sx+sy+sz) {
		puts("ABORTING: more than one source location !!");
		abort();
		}
	} //endif

//...find maximum amplitude time sample for trace jrec
	fmax_min(npts,tpoint,&FMAX,&FMIN,isw1,&KMX,&KMN);
	kmaxs1[jrec]=(float) KMX; 


     } //  end trace loop-----------------------------


//=============================================================================

//...allocate Freq[] array 
	tmax2=npr2*fsamin;
	delf=1.0/tmax2;
	fmin=delf*5.0;
	Nfreq=(int) (fmax-fmin)/delf;

	Freq=calloc(Nfreq,sizeof(double));
	if (Freq==NULL)
	{
          fprintf(stderr,"ABORT:Freq   can't allocate enough memory");
	  goto exit_2;
	} //endif


	/*  BUILD MODELS TO COMPUTE SYNTHETIC DATA TO COMPARE TO FILE DATA*/


	/* velocity loop */	
	for (j=0; j<(NV+1); j++) {


//      ID Number increment
//	Beta[0]=Vmn + (double)j*DV;
	Beta[0]=Vo;
	zi[0]=0.0;
	rho[0]=Rho ;
	CalcAlphaMu(Beta[0],rho[0],VpVs,&Alpha[0],&mu[0],&lame[0]); 

		/* depth loop */
		for (k=0; k<(NZ+1); k++) {
//	      	ID Number increment
		IDcase = IDcase + 1;

		Beta[1]=Vmn + (double)j*DV;
		zi[1]=Zmn + (double)k*DZ;
		rho[1]=Rho;
	CalcAlphaMu(Beta[1],rho[1],VpVs,&Alpha[1],&mu[1],&lame[1]); 
			Beta[2]=Vhsp;
			zi[2]=Zhsp;
			rho[2]=Rho;
	CalcAlphaMu(Beta[2],rho[2],VpVs,&Alpha[2],&mu[2],&lame[2]); 

//...display progress bar
        exbar(Ncases,bar,ibar,invbar,IDcase);
//	fprintf(stderr,"%d",IDcase);


	/*write out values  */
	fprintf(h3,"ID Case = %4.4d \n",IDcase); 
	for (m=0; m<3; m++) fprintf(h3," zi[%d]=%lf ",m,zi[m]); 
	fprintf(h3,"\n");
	for (m=0; m<3; m++) fprintf(h3," Beta[%d]=%lf ",m,Beta[m]); 
	fprintf(h3,"\n");
	for (m=0; m<3; m++) fprintf(h3," Alpha[%d]=%lf ",m,Alpha[m]); 
	fprintf(h3,"\n");
	for (m=0; m<3; m++) fprintf(h3," rho[%d]=%lf ",m,rho[m]); 
	fprintf(h3,"\n");
	for (m=0; m<3; m++) fprintf(h3," mu[%d]=%lf ",m,mu[m]); 
	fprintf(h3,"\n");
	for (m=0; m<3; m++) fprintf(h3," lame[%d]=%lf ",m,lame[m]); 
	fprintf(h3,"\n");

	/*  FREQUENCY LOOP: COMPUTE DISPERSION OF UP TO 9 MODES  */
//...compute dispersion for current velocity model

	Freq[0]=fmin;
        fprintf(h3," fmin=%f  delf=%f tmax=%f   \n",fmin,delf,tmax  );
        fprintf(h3," fmax=%f  Nfreq=%d    \n",fmax,Nfreq);
	fprintf(h3," npts=%d  npr2=%d \n",npts,npr2);

	for (jt=1; jt<Nfreq; jt++) {
	Freq[jt]=Freq[jt-1]+delf; 
	}

	indx=0;	
	for (jt=0; jt<Nfreq; jt++) {
	freq=Freq[jt];
//...call dispersion phase velocity at current frequency
	rwv_(&nlay, rho, mu, lame, zi, &deltz, &freq, &maxmod, pvf); 
	if (DEBUG0==1) fprintf(stdout,"freq=%f  ",freq);
	for (mode=0; mode<9; mode++) {
	pv[indx]=pvf[mode];
	indx=indx+1;
	if (DEBUG0==1) {
  	fprintf(stdout," cv[%d]=%f ",mode,pvf[mode]);
  	fprintf(stdout,"\n");} //endif
	} //endif
	} //end frequency loop

	

//=============================================================================

	/* LOOP THROUGH CASES  */

	nrec=ntraces;
//... compute synthetic seismic Rayleigh waves
//       segsw=0.0; // do not output BSEGY data from call to waves()
//	 segsw=1.0; // do output BSEGY data from waves()
	Nxout=npr2*nrec;
	fprintf(h3," Nxout=%d \n",Nxout);

	int	jd;
	for (jd=0; jd<Nxout; jd++) {
	xout[jd]=0.0E0;
//	fprintf(stderr," %d xout=%E \n",jd,xout[jd]);
	} //end zerout

	stepz=50 ;
	wavsub_(&Nfreq,Freq,pv,&fsamin2,&firvsel,sxyz,&nrec,Rx,Ry,Rz,&nlay,
	  rho,mu,lame,zi,&Nxout,xout,&segsw,&IDcase);



//...compute innerproduct = field (*) synthetic (xout)

	jmin=tmin/fsamin;
	jmax=tmax/fsamin;
//	fprintf(stdout," DEBUG: jmin=%d  jmax=%d \n",jmin,jmax);
	
	meanAngle=0.0;
	meanKmaxdif = 0.0;

	for (jrec=0; jrec<ntraces; jrec++) {
		iprodsum = (double) 0.00;
		s1norm = (double) 0.00;
		xoutnorm = (double) 0.00;
		jt2 = 0;
		for (jt=jmin; jt<jmax; jt++) {
			indexj = jt +jrec*npts;
			index2 = jt +jrec*npr2;
			iprodsum = iprodsum + ((double) s1[indexj])*xout[index2];
			s1norm = s1norm + ((double) s1[indexj])*((double) s1[indexj]);
			xoutnorm = xoutnorm + xout[index2]*xout[index2];
			s2[jt2] = xout[index2];
			jt2 = jt2 + 1;
		} // end sample loop for trace

//...find maximum amplitude time sample for trace jrec
	index2 = jmin +jrec*npr2;
	fmax_min(npts,s2,&FMAX,&FMIN,isw1,&KMX,&KMN);
	kmaxdif[jrec] = fabs((float)KMX - kmaxs1[jrec]);

	s1norm = sqrt(s1norm);
	xoutnorm = sqrt(xoutnorm);
	cosangle = iprodsum/(s1norm*xoutnorm);
/*      following line prevents cosangle >1.0 which can happen with an exact match
        between field and synthetic traces.  Sometimes, during testing, 
	a round off causes a domain error on the acos argument, why ????
	The over unity is very very small, and a nan results because of it.  */
	if (cosangle > (double) 1.0) cosangle = (double) 1.0000000000;
	if (cosangle < (double) -1.0) cosangle = (double) -1.0000000000;
	angle = acos(cosangle)*(((double) 180.)/PI);
	savangle[jrec]=angle;	
	meanAngle = meanAngle + angle;
	meanKmaxdif = meanKmaxdif + kmaxdif[jrec];
	fprintf(h3,"\t \t      Angle[%d]=%lf kmaxdif[%d]=%1f  \n",jrec,angle,jrec,kmaxdif[jrec]);
	fprintf(h3,"\t \t%2.2d  s1norm=%E \t xoutnorm=%E \n",IDcase,s1norm,xoutnorm);
	} //end trace loop

//...compute standard deviation (objective function)
	meanAngle = meanAngle/((float) ntraces);
	meanKmaxdif = meanKmaxdif/((float) ntraces);
	stdevAngle=0.0;
	for (jrec=0; jrec<ntraces; jrec++) {
	DELT=(meanAngle-savangle[jrec]);
	stdevAngle = stdevAngle + DELT*DELT;
	} //next jrec
	stdevAngle = sqrt(stdevAngle/((float) ntraces));
	savmean[IDcase]=meanAngle;
	savstdv[IDcase]=stdevAngle;
	savKZ[IDcase]=zi[1];
	savJV[IDcase]=Beta[1];
	savKMXdif[IDcase]=(meanKmaxdif);

	if (obselect == 1) 
	OBJF[IDcase]=stdevAngle + meanKmaxdif;
	else
	OBJF[IDcase]=stdevAngle;

	fprintf(h3,"\t \t | Mean Angle = %.2f StdDev = %.2f | Mean Kmaxdif = %.2f | ObjFun = %.2f \n",
	meanAngle,stdevAngle,meanKmaxdif,OBJF[IDcase]);
	fprintf(h3,"------------------------------------------------------\n");

//...capture maximum objective function
	if(OBJF[IDcase] > maxobjf) {
	maxobjf = OBJF[IDcase];
	maxcase = IDcase;
	}  //endif

//...capture minimum objective function 
		if(OBJF[IDcase] < minobjf) {
		minobjf = OBJF[IDcase];
		mincase = IDcase;
//save case parameters at minimum objective function	
	for (m=0; m<3; m++) svzi[m] = zi[m]; 
	for (m=0; m<3; m++) svBeta[m] = Beta[m]; 
	for (m=0; m<3; m++) svrho[m] = rho[m]; 
	for (m=0; m<3; m++) svmu[m] = mu[m]; 
	for (m=0; m<3; m++) svlame[m] = lame[m]; 
	 } //endif capture test


//DEBUG---Select one IDcase to output as a text file (compare to *.seg)
// set following to 0 and recompile if you want to activate this.
	int	skipdebugtxt=1;
	if (skipdebugtxt==0) {
// then set following to the number of case to be output as debug.txt file.
// example shown here is case 9 (only makes sense if at least 9 cases or more)
	if (IDcase==9) {

//...open output listing file
	FILE	*h4;
        if( (h4=fopen("debug.txt","w"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file debug.txt"); 
          goto quick_exit;
        }

	int	indexdebug;
			
		for(jt=0; jt<npts; jt++) {
		for (jrec=0; jrec<ntraces; jrec++) {
		indexdebug=jt +jrec*npr2;
	//	fprintf(stderr," \t%d \t%d \t%d\n",indexdebug,jt,jrec);
		fprintf(h4,"   %le ",xout[indexdebug]);
		} //next jrec
		fprintf(h4," \n");
		}  //next jt
	fclose(h4);
	}
   }
 //END DEBUG

// Output model of each case

	ModelOut(Beta,zi,IDcase);

		} //next k END OF DEPTH SCAN VELOCITY GRID

	} //next j  END OF VELOCITY MODEL LOOP 

//summary of findings

	fprintf(h3,"Number of Cases = %d \n",Ncases);
	fprintf(h3,"\tIDcase \tMean Angle \tStdDev \tKmaxdif \tZ[1] \tVs[1] \tOBJF \n");
	fprintf(h3,"\t       \t(degrees)  \t(deg)  \t(sample) \t(m)  \t(m/s) \n");
	for (jcase=0; jcase<Ncases; jcase++) {
	fprintf(h3,"\t  %d \t  %6.2f \t %.2f \t%.2f \t\t%.2f \t%.2f \t%.2f \n",
	jcase,savmean[jcase],savstdv[jcase],savKMXdif[jcase],savKZ[jcase],savJV[jcase],OBJF[jcase]);
	} //end jcase

//...Generate a gnuplot contour plot file
//...open output file
	char	ofileplot[13];
	sprintf(ofileplot,"contrplt.gp");
        if( (h5=fopen(ofileplot,"w"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s",ofileplot);
          goto exit_1;
        }
//...Generate Gnuplot file
	ContourPlot(h5,mincase,maxcase,Ncases,savmean,savstdv,savKZ,savJV,Zmn,Zmx,OBJF,NZ,infile,argc,argv);
	fclose(h5); //need to close for system command to work
	system("gnuplot -p contrplt.gp\n");

//...Generate a gnuplot contour plot file
//...open output file
	char	ofileplot2[13];
	sprintf(ofileplot2,"gridpoints.gp");
        if( (h5=fopen(ofileplot2,"w"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s",ofileplot2);
          goto exit_1;
        }
//...Generate Gnuplot file
	GridPoints(h5,mincase,maxcase,Ncases,savKZ,savJV,Vo,Vmn,Vmx,NV,
	Zmn,Zmx,NZ,Vhsp,Zhsp,DV,DZ,infile,argc,argv);
	fclose(h5); //need to close for system command to work
  	system("gnuplot -p gridpoints.gp\n");

// write out a disper.d file for minimum objective function case
	DisperdOut(svrho,svmu,svlame,svzi,deltz,Nfreq,fmin,delf,mincase);
// initial model for an octave inversion (3 layers, shear Velocity, depth to top)
	ModelOut(svBeta,svzi,mincase);

	fprintf(h3,"\n Maximum Case = %d  Maximum Objective Function= %g \n",maxcase,maxobjf);
	fprintf(stdout,"\n Maximum Case = %d  Maximum Objective Function= %g \n",maxcase,maxobjf);
	fprintf(h3,"\n Minimum Case = %d  Minimum Objective Function= %g \n",mincase,minobjf);
	fprintf(stdout,"\n Minimum Case = %d  Minimum Objective Function= %g \n",mincase,minobjf);

// write out a waves.d file for minimum objective function case
	WavesdOut(ntraces,Rx,Ry,Rz,sx,sy,sz,fmin,fmax,fsamin,firvsel,mincase,stepz);

	
//=============================================================================


      printf("\n   Listing ====>%s",lstfile);
//      printf("\n   Output  ====>%s",ofile);
      printf("\n   Processed %d cases of %d traces \n",Ncases,ntraces);
	free(s1);
	free(savangle);
	free(savmean );
	free(savstdv);
	free(savKZ   );
	free(savJV   );
	free(OBJF    );
	free(xout);
	free(Freq);
	free(kmaxs1);
	free(kmaxdif);
	free(savKMXdif);
//	free(s2);
      return(0);

quit_it:
        puts(" ");
//        free(s2);
exit_s1:
        free(s1);
exit_2:
	free(savangle);
	free(savmean );
	free(savstdv);
	free(savKZ   );
	free(savJV   );
	free(kmaxs1);
	free(savKMXdif);
	free(OBJF);
	free(kmaxdif);
        fclose(h2);
exit_1:
        fclose(h1);
exit_3:
        fclose(h3);
quick_exit:
          return(1);
     } //  end of main
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <paulmichaels@boisestate.edu>          |
   | FUNCTION NAME: getparm.c                                   |
   | Copyright (C) 2021 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: gets input parameters from command                |
   |          line or prompts user from consol.                 |
   |                                                            |
   |                                                            |
   |------------------------------------------------------------|
*/
      int  getparm(int argc,char *argv[],
	double *Vo, double *Vmn, double *Vmx, int *NV, double *Zmn, double *Zmx, int *NZ,
	double *Vhsp, double *Zhsp, double *VpVs, double *Rho, double *deltz, double *segsw,
	double *fmin, double *fmax, double *firvsel, float *tmin, float *tmax,  
	char infile[], char ofile[], char lstfile[],char pid[])
     {
      char              sfx1[4]="seg",
                        sfx2[4]="lst";
      char		help[3]="-h";
      char              msg1[46],msg2[46];

//...test for command line help ?
      if(argc==2 && !strcmp(argv[1],help))
     {
//...screen header and copyright notice
      cr_labl(pid);

//...help pannel
      sprintf(msg1,"Computes Rayleigh Wave Field Scan            ");
      sprintf(msg2,"and Objective Function for Waveform Inversion");
      hlp_labl(msg1,msg2);
      puts(" ");

//...help display
      puts(" ");
      printf("  %s  infile   Vo Vmn Vmx NV Zmn Zmx Nz \n",pid);
      printf("                 Vhsp Zhsp VpVs Rho deltz segsw \n");
      printf("                 fmin fmax firvsel tmin tmax   \n");
      puts(" ");
      puts("     infile  = input field data (4char minimum)");
      puts("                       FIXED                   "   );
      puts("     Vo     = S-velocity at surface      (double) ");
      puts("                       SCAN                       ");
      puts("     Vmn    = Minimum S-velocity scan    (double) ");
      puts("     Vmx    = Maximum S-velocity scan    (double) ");
      puts("     NV     = Number of Vs grid steps     (int)  ");
      puts("     Zmn    = Minimum Depth scan         (double)");
      puts("     Zmx    = Maximum Depth scan         (double) ");
      puts("     NZ     = Number of depth grid steps  (int)  ");
      puts("                       FIXED                     ");
      puts("     Vhsp   = Velocity of the half-space (double)");
      puts("     Zhsp   = Depth top of the half-space (double)");
      puts("     VpVs   = P to S wave velocity ratio (double) ");
      puts("     Rho    = Mass density               (double) ");
      puts("     deltz  = Depth integration step     (double) ");
      puts("     segsw  = Switch=1 output BSEGY files(double) ");
      puts("              Switch=0 No Output BSEGY   (double) ");
      puts("  Note: Octave *.m files always output             ");
      puts("     fmin   = minimum frequency Hz (example 1.0)   ");
      puts("     fmax   = maximum frequency Hz (example 100.0  ");
      puts("  NOTE: if fmin=1., fmax=100. expect minimum phase ");
      puts("  wavelet with 3db bandwidth 4-60 Hz ");
      puts("  and no amplitude < 1.0 Hz or > 100.0 Hz");
      puts("     firvsel = select component geophone ");
      puts("           0. = vertical ");
      puts("           1. = horizontal ");
      puts("     tmin = start time for time window in seconds ");
      puts("     tmax = end time for time window in seconds ");
      puts("     (trace samples outside window ignored ");
      puts("      when computing inner products and angles) ");
      puts(" ");
      puts(" ");
      return(1);
     }
      else
     {
//...get parameters------------------------------
//...pars off character parameters
      if(argc>1)
       strncpy(infile,argv[1],13);
       else
      {
       puts(" Enter INPUT file name ");
       scanf(" %13s",infile);
      }
	if (namecheck(12,infile) != 0 ) return (1);

//...convert string to float  
      if(argc>2)
       *Vo=atof(argv[2]);
       else
      {
       puts(" Enter S-velocity at Surface");
       scanf(" %lf",Vo);
      }

//...convert string to float  
      if(argc>3)
       *Vmn=atof(argv[3]);
       else
      {
       puts(" Enter minimum S-velocity scan");
       scanf(" %lf",Vmn);
      }

//...convert string to float  
      if(argc>4)
       *Vmx=atof(argv[4]);
       else
      {
       puts(" Enter maximum S-velocity scan");
       scanf(" %lf",Vmx);
      }

//...convert string to integer
      if(argc>5)
       *NV=atoi(argv[5]);
       else
      {
       puts(" Enter number of Vs grid steps");
       scanf(" %d",NV);
      }

//...convert string to float  
      if(argc>6)
       *Zmn=atof(argv[6]);
       else
      {
       puts(" Enter minimum Depth scan");
       scanf(" %lf",Zmn);
      }

//...convert string to float  
      if(argc>7)
       *Zmx=atof(argv[7]);
       else
      {
       puts(" Enter maximum Depth scan");
       scanf(" %lf",Zmx);
      }

//...convert string to integer
      if(argc>8)
       *NZ=atoi(argv[8]);
       else
      {
       puts(" Enter number of depth grid steps");
       scanf(" %d",NZ);
      }

//...convert string to float  
      if(argc>9)
       *Vhsp=atof(argv[9]);
       else
      {
       puts(" Enter Velocity of the half-space");
       scanf(" %lf",Vhsp);
      }

//...convert string to float  
      if(argc>10)
       *Zhsp=atof(argv[10]);
       else
      {
       puts(" Enter Depth to top of Half Space  ");
       scanf(" %lf",Zhsp);
      }

//...convert string to float  
      if(argc>11)
       *VpVs=atof(argv[11]);
       else
      {
       puts(" Enter P- to S-velocity ratio, VpVs");
       scanf(" %lf",VpVs);
      }

//...convert string to float  
      if(argc>12)
       *Rho=atof(argv[12]);
       else
      {
       puts(" Enter Mass Density, Rho");
       scanf(" %lf",Rho);
      }

//...convert string to float  
      if(argc>13)
       *deltz=atof(argv[13]);
       else
      {
       puts(" Enter integration depth step between control");
       scanf(" %lf",deltz);
      }

//...test to make sure Zmn > 0 (can't be same as surface control)
	if (*Zmn < *deltz) {
	*Zmn = *deltz;
	}

//...convert string to float  
      if(argc>14)
       *segsw=atof(argv[14]);
       else
      {
       puts(" Enter switch: 0=No BSEGY output 1=BSEGY out ");
       scanf(" %lf",segsw);
      }

//...convert string to float  
      if(argc>15)
       *fmin=atof(argv[15]);
       else
      {
       puts(" Enter minimum frequency Hz ");
       scanf(" %lf",fmin);
      }

//...convert string to float  
      if(argc>16)
       *fmax=atof(argv[16]);
       else
      {
       puts(" Enter maximum frequency Hz ");
       scanf(" %lf",fmax);
      }

//...convert string to float  
      if(argc>17)
       *firvsel=atof(argv[17]);
       else
      {
       puts(" Enter geophone component ");
       puts(" 0 = vertical component ");
       puts(" 1 = horizontal component ");
       scanf(" %lf",firvsel);
      }

//...convert string to float  
      if(argc>18)
       *tmin=atof(argv[18]);
       else
      {
       puts(" Enter start time in seconds for trace window ");
       scanf(" %f",tmin);
      }

//...convert string to float  
      if(argc>19)
       *tmax=atof(argv[19]);
       else
      {
       puts(" Enter end time in seconds for trace window ");
       scanf(" %f",tmax);
      }

//...name output files
      names(infile,pid,ofile,sfx1);
      names(infile,pid,lstfile,sfx2);
       return(0);
     }  //  endif bhelp test
      } // end getparm
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <paulmichaels@boisestate.edu>          |
   | FUNCTION NAME: outlst.c                                    |
   | Copyright (C) 2021 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: prints copyright header, file names, and          |
   |          echo check of input parameters in newly           |
   |          opened output listing file.                       |
   |                                                            |
   |------------------------------------------------------------|
*/
      void outlst(double Vo, double Vmn, double Vmx, int NV, double Zmn, 
	double Zmx, int NZ, double Vhsp, double Zhsp, double VpVs, 
	double Rho, double deltz, double segsw,
	double fmin, double fmax, double firvsel, float tmin, float tmax, 
	short obselect, char infile[], char pid[], char ofile[],
	char lstfile[], FILE *h3)
     {

//...print copyright label to outlst and screen

      fcr_labl(pid,h3);

//...echo check
      fputs("\n",h3);                      //blank line
      fprintf(h3,"\n     PROGRAM: %s.c ",pid);
      fputs("\n",h3);                      //blank line
      fputs("\n     FILES:       ",h3);
      fprintf(h3,"\n     Input   File= %s",infile);
//      fprintf(h3,"\n     Output  File= %s",ofile);
      fprintf(h3,"\n     Listing File= %s",lstfile);
      fputs("\n",h3);                      //blank line
      fputs("\n     INPUT PARAMETERS: \n",h3);
	fprintf(h3,"Vo  = %.1f \n",Vo );
	fprintf(h3,"Vmn = %.1f \n",Vmn);
	fprintf(h3,"Vmx = %.1f \n",Vmx);
	fprintf(h3,"NV  = %d \n",NV);
	fprintf(h3,"Zmn = %.1f \n",Zmn);
	fprintf(h3,"Zmx = %.1f \n",Zmx);
	fprintf(h3,"NZ  = %d \n",NZ);
	fprintf(h3,"Vhsp = %.1f \n",Vhsp);
	fprintf(h3,"Zhsp = %.1f \n",Zhsp);
	fprintf(h3,"VpVs = %.1f \n",VpVs);
	fprintf(h3,"Rho  = %.1f \n",Rho );
	fprintf(h3,"deltz= %.1f \n",deltz);
	fprintf(h3,"segsw= %.0f \n",segsw);
	fprintf(h3,"fmin = %.2f Hz \n",fmin );
	fprintf(h3,"fmax = %.2f Hz \n",fmax );
	if (segsw > 0.0001) {
	fprintf(h3,"BSEGY seismic traces OUTPUT \n");}
	else {
	fprintf(h3,"BSEGY seismic traces NOT OUTPUT \n");}
	if (firvsel > 0.0001) {
	fprintf(h3," Horizontal (radial) component geophones \n");}
	else {
	fprintf(h3," Vertical component geophones \n");}
	fprintf(h3,"tmin = %.4f (sec.) \n",tmin );
	fprintf(h3,"tmax = %.4f (sec.) \n",tmax );
	if (obselect == 1) {
	fprintf(h3,"Objective Fuction Definition (adjust for non-overlap): \n");
	fprintf(h3,"OBJF = stdevAngle + meanKmaxdif )   \n"); }
	else {
	fprintf(h3,"Objective Fuction Definition: \n");
	fprintf(h3,"OBJF = stdevAngle   \n"); 
	} //endif
      fputs("\n",h3);                      //blank line
     }  //  end outlst


	int CalcAlphaMu(double Vs,double Den, double VpVs, 
	 double *Vp, double *Mu, double *Lame )
	{
	*Vp = VpVs*Vs;
	*Mu = Den*Vs*Vs;
	*Lame = Den*( (*Vp)*(*Vp)-VpVs*Vs*Vs );
	return(0);
	} //end CalcAlphMu

// |--------------------------------------------------------------------------|
// | DisperOut Function                                                       |
// |--------------------------------------------------------------------------|
	void  DisperdOut(double svrho[], double svmu[], double svlame[],
	double svzi[], double deltz, int Nfreq, double fmin, double delf,
	int mincase) {
	FILE	*h4;
	short	m;
	char	outfile[20];
//...open output file
	sprintf(outfile,"disper%4.4d.d",mincase);
        if( (h4=fopen(outfile,"w"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file disperMin.d \n");
          abort ();
        }
	fprintf(h4,"  &disper   \n"   );	
	fprintf(h4,"  nlay=  3,    \n"   );	
	fprintf(h4,"  rho= %10.4E, ",svrho[0] );	
	for (m=1; m<3 ; m++) fprintf(h4,"%10.4E, ",svrho[m]);	
	fprintf(h4,"\n  mu= %10.4E, ",svmu[0] );	
	for (m=1; m<3 ; m++) fprintf(h4,"%10.4E, ",svmu[m]);	
	fprintf(h4,"\n  lame=%10.4E, ",svlame[0] );	
	for (m=1; m<3 ; m++) fprintf(h4,"%10.4E, ",svlame[m]);	
	fprintf(h4,"\n  zi=%10.4E, ",svzi[0] );	
	for (m=1; m<3 ; m++) fprintf(h4,"%10.4E, ",svzi[m]);	
	fprintf(h4,"\n  deltz=%10.3f,   \n",deltz);	
	fprintf(h4,"    modemx=9,   \n");	
	fprintf(h4,"    nfreq=%d, flo=%12.8E, delf=%12.8E, jsmax=300, ksw=0, \n",Nfreq,fmin,delf);	
	fprintf(h4,"     pvlcty=0.0, pfreq=0.0, zend=100.0,   \n");	
	fprintf(h4,"   ofile='disper.lst' \n");	
	fprintf(h4,"   octav1='phase.m', octav2='mat2.m', \n");	
	fprintf(h4,"   curve='earth.crv', / \n");	
	fclose(h4);	
} //end DisperOut 

// |--------------------------------------------------------------------------|
// | ModelOut  Function                                                       |
// |--------------------------------------------------------------------------|
	void  ModelOut(double svBeta[], double svzi[], int mincase) {
	FILE	*h4;
	short	m;
	char	outfile[20];
//...open output file
	sprintf(outfile,"model%4.4d.txt",mincase);
        if( (h4=fopen(outfile,"w"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file model%4.4d.txt \n",mincase);
          abort ();
        }
	fprintf(h4,"3  \n" );	
	for (m=0; m<3; m++) fprintf(h4," %lf ",svBeta[m]);
	fprintf(h4," \n");
	for (m=0; m<3; m++) fprintf(h4," %lf ",svzi[m]);
	fprintf(h4," \n");
	fclose(h4);
} //end ModelOut

// |--------------------------------------------------------------------------|
// | WavesdOut Function                                                       |
// |--------------------------------------------------------------------------|
	void  WavesdOut(int ntraces, double Rx[], double Ry[], double Rz[],
	double Sx, double Sy, double Sz,
	double fmin, double fmax, float fsamin, double firvsel, 
	int mincase, int stepz) {
	FILE	*h4;
	short	m;
	char	outfile[20];
	int	irvsel;
	int	j;
	irvsel = (int) floor(firvsel + .0001);	
//...open output file
	sprintf(outfile,"waves%4.4d.d",mincase);
        if( (h4=fopen(outfile,"w"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file wavesMin.d \n");
          abort ();
        }
	fprintf(h4,"  &waves    \n"   );	
	fprintf(h4,"  ksw= 0, stepz=%d, \n",stepz);
	fprintf(h4,"  modes=1,2,3,4,5,6,7,8,9, \n");
	fprintf(h4,"  fmin=%10.4f , fmax=%10.4f, \n",fmin,fmax);
	fprintf(h4,"  fsamin=%10.6f, \n",fsamin);
	fprintf(h4,"  curve='earth.crv', \n");
	fprintf(h4,"  mapmat=0, \n");
	fprintf(h4,"  matlb1='matc.m',  scilb1='matc.sci',\n");
	fprintf(h4,"  matlb2='matu.m',  scilb2='matu.sci',\n");
	fprintf(h4,"  irvsel=%d, \n",irvsel);
	fprintf(h4,"  ofile='waves.lst', / \n");
	fprintf(h4,"  &source \n");
	fprintf(h4,"  tm= 0.0, 0.0, 0.0, \n");
	fprintf(h4,"   0.0, 0.0, 0.0, \n");
	fprintf(h4,"   0.0, 0.0, -1.0,\n");  //default is Vertical Impact
	fprintf(h4,"  sx= %f sy= %f sz= %f, / \n",Sx,Sy,Sz);
	fprintf(h4,"  &recvr \n");
	fprintf(h4,"  nrec= %d \n",ntraces);
	fprintf(h4,"  rz= ");
	for (j=0; j<ntraces; j++) {
	fprintf(h4,"%f, ",Rz[j]);
	} //next j
	fprintf(h4,"  \n");
	fprintf(h4,"  ry= ");
	for (j=0; j<ntraces; j++) {
	fprintf(h4,"%f, ",Ry[j]);
	} //next j
	fprintf(h4,"  \n");
	fprintf(h4,"  rx= ");
	for (j=0; j<ntraces; j++) {
	fprintf(h4,"%f, ",Rx[j]);
	} //next j
	fprintf(h4,"  \n");
	fprintf(h4," / \n");
	fclose(h4);
	} //end WavesdOut
// |--------------------------------------------------------------------------|
// | ContourPlot                                                              |
// |--------------------------------------------------------------------------|
	void	ContourPlot(FILE * h5, int mincase, int maxcase, int Ncases, 
	double savmean[], double savstdv[], double savKZ[], double savJV[], 
	double Zmn, double Zmx,
	double OBJF[], int NZ, char infile[],int argc,char *argv[])
	{
	int 	jcase, j;
	int	result;
	char	fmt[]="%.1f";
	fprintf(h5,"#Generated by program bwfi.c  \n");
	fprintf(h5,"# ");
	for (j=0; j<argc; j++) fprintf(h5," %s",argv[j]);
	fprintf(h5," \n");
	fprintf(h5,"#Grid Search Wave Form Inversion  \n");
	fprintf(h5,"# mincase=%d maxcase=%d \n",mincase,maxcase);
	fprintf(h5,"#set terminal postscript eps size 6.5,6.0 enhanced color font \"Helvetica,20\" \n");
	fprintf(h5,"#set output \"solutionspace.eps\" \n");
	fprintf(h5,"set grid \n");
	fprintf(h5,"set mxtics; set mytics;  \n");
	fprintf(h5,"set ylabel \"Moving Point Depth (m)\" \n");
	fprintf(h5,"set xlabel \"Moving Point Vs (m/s)\" \n");
	fprintf(h5,"set yrange [%f:%f] \n",Zmx,Zmn);
	fprintf(h5,"set title \"Objective Function For File:     %s   \" \n",infile);
	fprintf(h5,"unset surface \n");
	fprintf(h5,"set multiplot \n");
	fprintf(h5,"TYPE=\"color\"  \n");
	fprintf(h5,"#TYPE=\"contour\"  \n");
	fprintf(h5,"if (TYPE ne \"color\") {  \n");
	fprintf(h5,"set view map \n");
	fprintf(h5,"set contour \n");
	fprintf(h5,"# comment next line for contour color key   \n");
	fprintf(h5,"set nokey  \n");
	fprintf(h5,"set clabel  \" %s \" \n",fmt);
	fprintf(h5,"# comment following to give colored contours  \n");
	fprintf(h5,"unset clabel  \n");
	fprintf(h5,"set cntrparam bspline \n");
	fprintf(h5,"set cntrparam levels 10 \n");
	fprintf(h5,"} else {  \n");
	fprintf(h5,"set pm3d \n");
	fprintf(h5,"set view map \n");
	fprintf(h5,"set nokey \n");
	fprintf(h5,"# uncomment next line for interpolation to a 60x60 grid \n");
	fprintf(h5,"#set dgrid3d 60, 60, gauss 0.6 0.6 \n");
	fprintf(h5,"set palette model RGB defined (0.0 'violet', .20 'blue',  .40 'green', ");
	fprintf(h5,".60 'yellow', .80 'orange', 1.00 'red') \n");
	fprintf(h5,"}  \n");
	fprintf(h5,"splot '-' u 5:4:6 w lines t \"stdv\" lc \"black\" lw 1 \n");
	fprintf(h5,"#Number of Cases=%d  \n",Ncases);
	fprintf(h5,"#	IDcase 	Mean Angle 	StdDev 	Z[1] 	Vs[1]  OBJF  \n");
	fprintf(h5,"#	       	(degrees)  	(deg)  	(m)  	(m/s)        \n");
	for (jcase=0; jcase<Ncases; jcase++) {
	result = (jcase+NZ+1) % (NZ+1);
	if ( result == 0 ) {
	fprintf(h5,"  \n");
	} //endif
	fprintf(h5,"\t  %d \t  %6.2f \t %.2f \t%2.2f \t%.2f \t%.2f \n",
	jcase,savmean[jcase],savstdv[jcase],savKZ[jcase],savJV[jcase],OBJF[jcase]);
	} //end jcase
  	fprintf(h5,"e\n");  //this is required when std in - plot

	fprintf(h5,"set label front at %f, %f \"  min\" tc \"black\" point pt 7 ps 1 lc rgb \"black\" \n",
	savJV[mincase],savKZ[mincase]);
	fprintf(h5,"set label front at %f, %f \"  max\" tc  \"black\" point pt 7 ps 1 lc rgb \"black\" \n",
	savJV[maxcase],savKZ[maxcase]);
	fprintf(h5,"replot \n"); 
//now contours as multiplot
	fprintf(h5,"#TYPE=\"color\"  \n");
	fprintf(h5,"TYPE=\"contour\"  \n");
	fprintf(h5,"if (TYPE ne \"color\") {  \n");
	fprintf(h5,"set view map \n");
	fprintf(h5,"set contour \n");
	fprintf(h5,"# comment next line for contour color key   \n");
	fprintf(h5,"set nokey  \n");
	fprintf(h5,"set clabel  \" %s \" \n",fmt);
	fprintf(h5,"# comment following to give colored contours  \n");
	fprintf(h5,"unset clabel  \n");
	fprintf(h5,"set cntrparam bspline \n");
	fprintf(h5,"set cntrparam levels 10 \n");
	fprintf(h5,"} else {  \n");
	fprintf(h5,"set pm3d \n");
	fprintf(h5,"set view map \n");
	fprintf(h5,"set nokey \n");
	fprintf(h5,"# uncomment next line for interpolation to a 60x60 grid \n");
	fprintf(h5,"#set dgrid3d 60, 60, gauss 0.6 0.6 \n");
	fprintf(h5,"set palette model RGB defined (0.0 'violet', .20 'blue',  .40 'green', ");
	fprintf(h5,".60 'yellow', .80 'orange', 1.00 'red') \n");
	fprintf(h5,"}  \n");
	fprintf(h5,"splot '-' u 5:4:6 w lines t \"stdv\" lc \"black\" lw 1 \n");
	fprintf(h5,"#Number of Cases=%d  \n",Ncases);
	fprintf(h5,"#	IDcase 	Mean Angle 	StdDev 	Z[1] 	Vs[1]  OBJF  \n");
	fprintf(h5,"#	       	(degrees)  	(deg)  	(m)  	(m/s)        \n");
	for (jcase=0; jcase<Ncases; jcase++) {
	result = (jcase+NZ+1) % (NZ+1);
	if ( result == 0 ) {
	fprintf(h5,"  \n");
	} //endif
	fprintf(h5,"\t  %d \t  %6.2f \t %.2f \t%2.2f \t%.2f \t%.2f \n",
	jcase,savmean[jcase],savstdv[jcase],savKZ[jcase],savJV[jcase],OBJF[jcase]);
	} //end jcase
  	fprintf(h5,"e\n");  //this is required when std in - plot
	fprintf(h5,"unset multiplot \n");

	} //end ContourPlot

// |--------------------------------------------------------------------------|
// | Grid Point Plot                                                          |
// |--------------------------------------------------------------------------|
	void	GridPoints(FILE * h5, int mincase, int maxcase, int Ncases, 
	double savKZ[], double savJV[], double Vo, double Vmn, double Vmx, int NV,
	double Zmn, double Zmx, int NZ, double Vhsp, double Zhsp, double DV, double DZ, 
	char infile[],int argc,char *argv[])
	{
	int 	j;
	char	fmt[]="%.1f";
	fprintf(h5,"#GridPoints: Generated by program bwfi.c  \n");
	fprintf(h5,"# ");
	for (j=0; j<argc; j++) fprintf(h5," %s",argv[j]);
	fprintf(h5," \n");
	fprintf(h5,"#Grid Points in Search Wave Form Inversion  \n");
	fprintf(h5,"# mincase=%d maxcase=%d \n",mincase,maxcase);
	fprintf(h5,"#set terminal postscript eps size 6.5,6.0 enhanced color font \"Helvetica,20\" \n");
	fprintf(h5,"#set output \"solutionspace.eps\" \n");
	fprintf(h5,"set nokey \n");
	fprintf(h5,"set grid \n");
	fprintf(h5,"set mxtics; set mytics;  \n");
	fprintf(h5,"set ylabel \"Depth (m)\" \n");
	fprintf(h5,"set xlabel \"Vs (m/s)\" \n");
	fprintf(h5,"set yrange [%f:0.0] \n",Zmx+1.0);
	fprintf(h5,"set xrange [0.0:%f] \n",Vmx+100.0);
	fprintf(h5,"set title \"Grid Points in Search  \" \n");
//
//...main loop
	fprintf(h5,"plot '-' u 2:1 w p pt 7 lc 'red' , \\\n");
	fprintf(h5," '-'  u 2:1 w l lc 'red' lw 3 , \\\n");
	fprintf(h5," '-' u 2:1 w p pt 7 ps 2 lc 'blue', \\\n");
	fprintf(h5," '-' u 2:1 w l lc 'red' lw 3 , \\\n");
	fprintf(h5," '-' u 2:1 w p pt 7 ps 2 lc 'blue' \n");
	for (j=0; j<Ncases; j++) {
	fprintf(h5," %f \t %f \n",savKZ[j],savJV[j]);
	} //next j
	fprintf(h5,"e \n");
	fprintf(h5," 0.0  0.0 \n");
	fprintf(h5," 0.0  %f  \n",Vo);
	fprintf(h5,"e \n");
	fprintf(h5," 0.0  %f  \n",Vo);
	fprintf(h5,"e \n");
	fprintf(h5," %f  0.0 \n",Zhsp);
	fprintf(h5," %f  %f \n",Zhsp,Vhsp);
	fprintf(h5,"e \n");
	fprintf(h5," %f  %f \n",Zhsp,Vhsp);
  	fprintf(h5,"e\n");  //this is required when std in - plot
	fprintf(h5,"set label front at %f, %f \"  min\" tc \"black\" point pt 7 ps 1 lc rgb \"black\" \n",
	savJV[mincase],savKZ[mincase]);
	fprintf(h5,"set label front at %f, %f \"  max\" tc  \"black\" point pt 7 ps 1 lc rgb \"black\" \n",
	savJV[maxcase],savKZ[maxcase]);
	fprintf(h5,"replot \n"); 


	} //endGridPoints
